//
//  ConfigClass.swift
//  This is to provide a central location for handling configuration settings, such as for Google Ads
//  Fog of Text
//
//  Created by Joe Mccormack on 7/6/20.
//  Copyright © 2020 Joe Mccormack. All rights reserved.
//

import Foundation

class ConfigClass {
    /* Flag to indicate if the TOC has been accepted or not. Set to true for testing. */
    var tocAccepted: Bool = true
    /* Flag to indicate if in-app purchase (to unlock settings) has been completed.  Set to true for testing. */
    var inAppPurchaseCompleted: Bool = true
    /* Flag to indicate if the user has selected to use custom JSON.  User cannot change this selection until in-app purchase has been completed. Set to true for testing. */
    var usecustomjson: Bool = true
    
    /* Product ID of this app in app-store-connect */
    var appProductID = "unknown";

    /*
     The following IDs should be used, depending on environment, for the Banner Ad ID.  The Mobile Ads App ID (aka GADApplicationIdentifier) is defined as a key/value pair in the Info.plist file and does not need to be changed based on the environment.
     TESTING    - ca-app-pub-???
     PRODUCTION - ca-app-pub-???
     */
    //var googleBannerAdUnitID = "ca-app-pub-???"
    /* Flag incidating whether to show or not show ads */
    var showAds = false
    
    /* Determine if user has made in-app purchase to not show ads by checking stored value on device */
    func hasPurchasedAdRemoval() {
        let purchaseCheck = UserDefaults.standard.bool(forKey: appProductID)
        if (purchaseCheck == true) {
            showAds = false
        }
    }
    /* Disables the showing of banner ads (user can complete in-app purchase to not show ads) */
    func disableShowBannerAds() {
        showAds = false
        /* Persist the purchased flag between app sessions */
        UserDefaults.standard.set(true, forKey: appProductID)
    }
    
    /* Saves that an in-app purchase has been completed */
    func saveInAppPurchaseCompleted() {
        UserDefaults.standard.set(true, forKey: "inapppurchasecompleted")
        inAppPurchaseCompleted = true
    }
    
    /* Loads whether the TOC has been accepted or not */
    func recallToc() {
        let defaults = UserDefaults.standard
        let tmpTocAccepted: Bool = defaults.bool(forKey: "tocaccepted")
        if tmpTocAccepted == true {
            tocAccepted = true
        }
        else {
            tocAccepted = false
        }
    }
    
    /* Saves that the TOC has been accepted */
    func saveTocAccepted() {
        UserDefaults.standard.set(true, forKey: "tocaccepted")
        tocAccepted = true
    }
    
    /* Loads whether an in-app purchase has been completed or not */
    func recallInAppPurchaseCompleted() {
        let defaults = UserDefaults.standard
        let tmpPurchaseCompleted: Bool = defaults.bool(forKey: "inapppurchasecompleted")
        if tmpPurchaseCompleted == true {
            inAppPurchaseCompleted = true
        }
        else {
            inAppPurchaseCompleted = false
        }
    }
    
    /* Get the current value of the "usecustomjson" in userdefaults.standard of this app */
    func recallUseCustomJsonValue() {
        let defaults = UserDefaults.standard
        let tmpUseCustomJson: Bool = defaults.bool(forKey: "usecustomjson")
        if tmpUseCustomJson == true {
            usecustomjson = true
        }
        else {
            usecustomjson = false
        }
    }
    
    /* Updates the state of "usecustomjson" in userdefaults.standard */
    func updateUseCustomJson(status: Bool) {
        if inAppPurchaseCompleted == true {
            let defaults = UserDefaults.standard
            if status == true {
                usecustomjson = true
                defaults.set(usecustomjson, forKey: "usecustomjson")
            }
            else {
                usecustomjson = false
                defaults.set(usecustomjson, forKey: "usecustomjson")
            }
        }
    }
    
}
