//
//  FirstViewController.swift
//  This controller focuses on encoding and decoding text.
//  Fog of Text
//
//  Created by Joe Mccormack on 7/5/20.
//  Copyright © 2020 Joe Mccormack. All rights reserved.
//

import UIKit
/* Google Ads - Importation */
/*
import GoogleMobileAds
*/

class FirstViewController: UIViewController, UITextFieldDelegate {
    /* Google Ads - Initialization for this controller; currently bypassed with a placeholder for future use */
    /*
    @IBOutlet weak var bannerView: GADBannerView!
    */
    @IBOutlet weak var bannerView: UILabel!
    
    /* Text field for user to type text */
    @IBOutlet weak var EncodeTextString: UITextField!
    /* Encoded value of text user typed */
    @IBOutlet weak var EncodedTextString: UITextView!
    /* Label to show the number of characters used in encoding */
    @IBOutlet weak var encodedCharacterCount: UILabel!
    /* Text field for user to paste encoded text to decode */
    @IBOutlet weak var DecodeTextString: UITextField!
    /* Decoded value of text user pasted */
    @IBOutlet weak var DecodedTextString: UITextView!

    /* When keyboard return key is pressed after being invoked by the Text field, this dismisses the keyboard.  When IBAction was created it was set with "Did End on Exit" */
    @IBAction func EncodeTextStringKeyboardReturn(_ sender: UITextField) {
        /* Do nothing because we have a separate button to handle the download process */
    }
    @IBAction func DecodeTextStringKeyboardReturn(_ sender: UITextField) {
        /* Do nothing because we have a separate button to handle the download process */
    }
    
    /* Button action to encode text */
    @IBAction func EncodeTextButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            encodedCharacterCount.text = ""
            var sampleString = EncodeTextString.text!
            if sampleString.count > 0 {
                var Obj_ProcessJsonDataClass = ProcessJsonDataClass()
                var encodedStr: String = Obj_ProcessJsonDataClass.encodeStr(rawText: sampleString)
                EncodedTextString.text = encodedStr
                encodedCharacterCount.text = String("\(encodedStr.count) characters in encoding.")
                /* Dismiss keyboard for encoding text in the event the return key was not pressed on the keyboard */
                EncodeTextString.endEditing(true)
            }
            else {
                /* Dismiss keyboard for encoding text */
                EncodeTextString.endEditing(true)
            }
        }
    }
    
    /* Button action to copy encoded text to clipboard */
    @IBAction func CopyEncodedButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            var sampleString: String = EncodedTextString.text!
            if sampleString.count > 0 {
                let pasteBoard = UIPasteboard.general
                pasteBoard.string = sampleString
                /* Dismiss keyboards if they are visible */
                EncodeTextString.endEditing(true)
                DecodeTextString.endEditing(true)
            }
        }
    }
    
    /* Button action to decode text that is encoded */
    @IBAction func DecodeTextButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            var sampleString: String = DecodeTextString.text!
            if sampleString.count > 0 {
                var Obj_ProcessJsonDataClass = ProcessJsonDataClass()
                var decodedStr: String = Obj_ProcessJsonDataClass.decodeStr(rawText: sampleString)
                DecodedTextString.insertText("\n" + decodedStr)
                /* Dismiss keyboards if they are visible */
                EncodeTextString.endEditing(true)
                DecodeTextString.endEditing(true)
            }
            else {
                /* Dismiss keyboard for decoding text */
                DecodeTextString.endEditing(true)
            }
        }
    }
    
    /* Clear button to remove text from encoding text areas */
    @IBAction func EncodeTextClearButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            EncodeTextString.text = ""
            EncodedTextString.text = ""
            encodedCharacterCount.text = ""
            /* Dismiss the keyboard if it is present */
            EncodeTextString.endEditing(true)
        }
    }
    
    /* Clear button to remove text from decoding text areas */
    @IBAction func DecodeTextClearButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            DecodeTextString.text = ""
            DecodedTextString.text = ""
            /* Dismiss the keyboard if it is present */
            DecodeTextString.endEditing(true)
        }
    }
    
    /* Alert error message to show if an error was found. */
    func foundError(msg: String, status: Bool) {
        /* Show alert message */
        let alertController = UIAlertController(title: "Error Found", message: msg, preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Terms and Conditions acceptance check */
    func didAcceptToc() -> Bool {
        var acceptedResult: Bool = false
        /* Reference to configuration data */
        var Obj_ConfigClass = ConfigClass()
        /* Check to see if TOC was accepted */
        Obj_ConfigClass.recallToc()
        /* TOC was accepted by the user */
        if Obj_ConfigClass.tocAccepted == true {
            acceptedResult = true
        }
        return acceptedResult
    }
    
    /* Terms and Conditions accept/decline handling functions */
    func showToc() {
        var returnState: Bool = true
        var tocMessage: String = ""
        /* Load TOC content to display */
        let tocFilename: String = "toc"
        if let tocurl = Bundle.main.url(forResource: tocFilename, withExtension: "txt") {
            do {
                /* Load Data */
                let tocData = try String(contentsOf: tocurl)
                /* Dump data into tocMessage */
                tocMessage = tocData
            }
            catch {
                returnState = false
                tocMessage = "Terms and Conditions could not be loaded: \(error.localizedDescription).  Immediately exit this software application, remove it and then re-install it.  If the problem persists, remove and do not re-install it."
            }
        }

        if returnState == true {
            /* Display the TOC to the user to accept/decline */
            tocMessageShow(msg: tocMessage)
        }
        else {
            /* Data could not be loaded, alert user */
            foundError(msg: tocMessage, status: returnState)
        }
    }
    
    /* Show TOC for user to accept/decline */
    func tocMessageShow(msg: String) {
        let alertController = UIAlertController(title: "Terms and Conditions", message: msg, preferredStyle: .alert)
        let acceptAction = UIAlertAction(title: "Accept", style: .default, handler: { action in
            /* User accepted TOC */
            var Obj_ConfigClass = ConfigClass()
            Obj_ConfigClass.saveTocAccepted()
            /* "Reload" the configuration and initializataions so that the objects in the view controller work without the user needing to switch to another view controller, then come back, or restart the app. */
            self.reloadConfigurations()
        })
        let declineAction = UIAlertAction(title: "Decline", style: .default, handler: { action in
            /* User declined TOC */
        })
        alertController.addAction(acceptAction)
        alertController.addAction(declineAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Since configuration setting is needed in multiple locations in this view controller because of the TOC acceptance requirement, those configurations are located in reloadConfigurations() */
    func reloadConfigurations() {
        /* Reference to configuration data */
        var Obj_ConfigClass = ConfigClass()
        /* Reference to the functions which handle JSON data */
        var Obj_HandleJsonDataClass = HandleJsonDataClass()
        
        /* Retrieve and set state of in-app purchase completion */
        Obj_ConfigClass.recallInAppPurchaseCompleted()
        
        /* Based on toggle state saved in userdefaults.standard, determine if default or custom json is utilized for this session */
        Obj_ConfigClass.recallUseCustomJsonValue()
        if Obj_ConfigClass.usecustomjson == true {
            /* Load the customjson.json from this app's Documents folder, if it is present.  If it is missing or invalid, alert user and load the default.json instead. */
            if (Obj_HandleJsonDataClass.loadJSONCustom() == true) {
                /* Custom JSON loaded for the app session */
            }
            else {
                /* Data could not be loaded, alert user and attempt to load the default.json instead */
                var tmpHasLoadedMsg: String = Obj_HandleJsonDataClass.hasLoadedMsg
                tmpHasLoadedMsg = tmpHasLoadedMsg + "  For now, the default JSON of this app will be loaded instead of the custom JSON."
                foundError(msg: tmpHasLoadedMsg, status: Obj_HandleJsonDataClass.hasLoaded)
                /* Load the default.json from this app's Resources folder */
                if (Obj_HandleJsonDataClass.loadJSONDefault() == true) {
                    /* Default JSON loaded for the app session */
                }
                else {
                    /* Data could not be loaded, alert user */
                    foundError(msg: Obj_HandleJsonDataClass.hasLoadedMsg, status: Obj_HandleJsonDataClass.hasLoaded)
                }
            }
        }
        else {
            /* Load the default.json from this app's Resources folder */
            if (Obj_HandleJsonDataClass.loadJSONDefault() == true) {
                /* Default JSON loaded for the app session */
            }
            else {
                /* Data could not be loaded, alert user */
                foundError(msg: Obj_HandleJsonDataClass.hasLoadedMsg, status: Obj_HandleJsonDataClass.hasLoaded)
            }
        }

        /* TESTING DATA */
        //EncodeTextString.insertText("hello there")
    }
    
    /* Special handling to show TOC for the user to accept/decline since alerts cannot be triggered from viewDidLoad().  Handy. */
    override func viewDidAppear(_ animated: Bool) {
        /* TOC was accepted by the user */
        if didAcceptToc() == true {
            reloadConfigurations()
        }
        else {
            /* TOC not accepted; provide TOC message for user to accept/decline */
            showToc()
        }
    }
    
    /* On load actions that need to be completed */
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        
        /* Google Ads - Configuration and invocation for this controller */
        /*
        Obj_ConfigClass.hasPurchasedAdRemoval()
        if (Obj_ConfigClass.showAds == true) {
            /* Show Ads */
            bannerView.adSize = kGADAdSizeBanner
            bannerView.adUnitID = Obj_ConfigClass.googleBannerAdUnitID
            bannerView.rootViewController = self
            bannerView.load(GADRequest())
            /* Uncomment for troubleshooting banner ad retrieval issues */
            //bannerView.delegate = self
        }
        */

        /* Delegate the UITextFields so that pressing the return key will cause the keyboard to dismiss.  This does not actually work so use the IBAction attached to those UITextfields instead. */
        //self.EncodeTextString.delegate = self
        //self.DecodeTextString.delegate = self
        
        /* Make the UITextViews non-editable, but selectable for copying content.  The keyboard is not needed for them. */
        EncodedTextString.isSelectable = true
        EncodedTextString.isEditable = false
        DecodedTextString.isSelectable = true
        DecodedTextString.isEditable = false
    }
}

/* Google Ads - Delegate primarily for troubleshooting */
/*
extension FirstViewController: GADBannerViewDelegate {
    func adViewDidReceiveAd(_ bannerView: GADBannerView) {
        print("Received Ad")
    }
    func adView(_ bannerView: GADBannerView, didFailToReceiveAdWithError error: GADRequestError) {
        print("Banner Retrival Error \(error)")
    }
}
 */
