//
//  SecondViewController.swift
//  This controller focuses on app settings.
//  Fog of Text
//
//  Created by Joe Mccormack on 7/5/20.
//  Copyright © 2020 Joe Mccormack. All rights reserved.
//

import UIKit
import StoreKit
/* Google Ads - Importation */
/*
import GoogleMobileAds
*/

class SecondViewController: UIViewController, UITextFieldDelegate, SKPaymentTransactionObserver {
    /* Google Ads - Initialization for this controller; currently bypassed with a placeholder for future use */
    /*
    @IBOutlet weak var bannerView: GADBannerView!
    */
    @IBOutlet weak var bannerView: UILabel!
    
    /* In-app purchase handling stub required for SKPaymentTransactionObserver.  This is the listener for the result of payment processing. */
    func paymentQueue(_ queue: SKPaymentQueue, updatedTransactions transactions: [SKPaymentTransaction]) {
        for transaction in transactions {
            if transaction.transactionState == .purchased {
                /* In-app purchase completed */
                /* Reference to configuration data */
                var Obj_ConfigClass = ConfigClass()
                /* Save successful in-app purchase with key "inapppurchasecompleted" */
                Obj_ConfigClass.saveInAppPurchaseCompleted()
                /* End transaction */
                SKPaymentQueue.default().finishTransaction(transaction)
            }
            else if transaction.transactionState == .failed {
                /* In-app purchase failed.  Tell user if there was an error. */
                /* End transaction */
                SKPaymentQueue.default().finishTransaction(transaction)
                /* Tell user if there was an error. */
                if let error = transaction.error {
                    let errorMsg = error.localizedDescription
                    inAppPurchaseError(paymentFailedMsg: errorMsg)
                }
            }
            else if transaction.transactionState == .restored {
                /* In-app purchase confirmed.  Restore purchase. */
                /* Reference to configuration data */
                var Obj_ConfigClass = ConfigClass()
                /* Restore in-app purchase */
                Obj_ConfigClass.saveInAppPurchaseCompleted()
                /* End transaction */
                SKPaymentQueue.default().finishTransaction(transaction)
            }
        }
    }
    
    /* Button to complete in-app purchase */
    @IBAction func unlockFeaturesButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            /* Reference to configuration data */
            var Obj_ConfigClass = ConfigClass()
            /* If in-app purchase has already been completed, block user from doing it again. */
            Obj_ConfigClass.recallInAppPurchaseCompleted()
            if Obj_ConfigClass.inAppPurchaseCompleted == false {
                /* Complete in-app purchase, if the user can complete a purchase */
                if SKPaymentQueue.canMakePayments() {
                    let paymentRequest = SKMutablePayment()
                    paymentRequest.productIdentifier = Obj_ConfigClass.appProductID
                    SKPaymentQueue.default().add(paymentRequest)
                }
                else {
                    /* Does not look like user can complete in-app purchase */
                    inAppPurchaseNotPossible()
                }
            }
            else {
                /* User has already completed in-app purchase, alert them they do not need to do it again. */
                inAppPurchaseAlreadyCompleted()
            }
        }
    }
    
    /* Button to restore in-app purchase */
    @IBAction func restoreInAppPurchaseButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            /* Check to see if in-app purchase was completed (as recorded at Apple) and, if so, restore the in-app purchase.  This is actually handled by the paymentQueue handler. */
            SKPaymentQueue.default().restoreCompletedTransactions()
        }
    }

    /* Text field for the user to enter an https URL for custom JSON download */
    @IBOutlet weak var downloadCustomJsonBox: UITextField!
    /* In order to access a property of the UISwitch outside of the IBAction (user interacting with it) there needs to also have a IBOutlet */
    @IBOutlet weak var useCustomJsonToggleManuallySet: UISwitch!
    /* When keyboard return key is pressed after being invoked by the Text field, this dismisses the keyboard.  When IBAction was created it was set with "Did End on Exit" */
    @IBAction func downloadCustomJsonBoxKeyboardReturn(_ sender: UITextField) {
        /* Do nothing because we have a separate button to handle the download process */
    }

    /* Button to restore the custom JSON file in the app's Documents folder */
    @IBAction func resetCustomJsonFileButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            /* Reference to configuration data */
            var Obj_ConfigClass = ConfigClass()
            Obj_ConfigClass.recallInAppPurchaseCompleted()
            if Obj_ConfigClass.inAppPurchaseCompleted == true {
                /* Reference to the functions which handle JSON data */
                var Obj_HandleJsonDataClass = HandleJsonDataClass()
                if Obj_HandleJsonDataClass.reseetJSONCustom() == true {
                    /* Operation Completed */
                    foundResetSuccess()
                }
                else {
                    /* Operation Failed */
                    foundError(msg: Obj_HandleJsonDataClass.hasSavedMsg, status: Obj_HandleJsonDataClass.hasSaved)
                }
            }
            else {
                /* In-app purchase not completed so setting is not available */
                settingNotEnabled()
            }
        }
    }
    
    /* Alert message for errors that may have been found. */
    func foundError(msg: String, status: Bool) {
        if status == false {
            /* Show alert message */
            let alertController = UIAlertController(title: "Error Found", message: msg, preferredStyle: .alert)
            let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
            alertController.addAction(defaultAction)
            present(alertController, animated: true, completion: nil)
        }
    }
    
    /* Switch to toggle usage of the custom JSON file on or off */
    /* NOTE: Well-known bug not yet fixed with SwiftUI and Xcode cause a false-positive error to be generated anytime the switch is used */
    @IBAction func useCustomJsonToggle(_ sender: UISwitch) {
        if didAcceptToc() == true {
            /* Reference to configuration data */
            var Obj_ConfigClass = ConfigClass()
            Obj_ConfigClass.recallInAppPurchaseCompleted()
            if Obj_ConfigClass.inAppPurchaseCompleted == true {
                if (sender.isOn == true) {
                    Obj_ConfigClass.updateUseCustomJson(status: true)
                }
                else {
                    Obj_ConfigClass.updateUseCustomJson(status: false)
                }
            }
            else {
                /* In-app purchase not completed so setting is not available */
                sender.isOn = false
                settingNotEnabled()
            }
        }
        else {
            sender.isOn = false
        }
    }
    
    /* Button that initiates the download of the custom JSON */
    @IBAction func downloadCustomJsonButton(_ sender: UIButton) {
        if didAcceptToc() == true {
            /* Reference to configuration data */
            var Obj_ConfigClass = ConfigClass()
            Obj_ConfigClass.recallInAppPurchaseCompleted()
            if Obj_ConfigClass.inAppPurchaseCompleted == true {
                /* Load JSON data from user - user will NOT be alerted on completion because the completion function in the external class cannot gain access to the function in this view controller's class called "foundError()" which presents the alert message.  Quaint. */
                var targetUrl:String = String(downloadCustomJsonBox.text!)
                if targetUrl.count > 13 {
                    /* Determine if URL is valid */
                    downloadCustomJsonButtonHelperResult = true
                    downloadCustomJsonButtonHelper(targetUrl: targetUrl)
                    if downloadCustomJsonButtonHelperResult == true {
                        /* Alert user that the action is about to take place */
                        mentionLoadStart()
                        /* Reference to the functions which handle JSON data */
                        var Obj_HandleJsonDataClass = HandleJsonDataClass()
                        /* Load JSON data asynchronously */
                        Obj_HandleJsonDataClass.loadJSONDataFromUrl(targetUrl: targetUrl)
                    }
                    else {
                        /* URL appears to be invalid */
                        mentionUrlInvalid()
                   }
                }
                else {
                    /* URL appears to be invalid */
                    downloadCustomJsonBox.text = "https://"
                    mentionUrlInvalid()
                }
            }
            else {
                /* In-app purchase not completed so setting is not available */
                settingNotEnabled()
            }
        }
    }
    /* Helper to determine if url is valid */
    var downloadCustomJsonButtonHelperResult: Bool = true
    func downloadCustomJsonButtonHelper(targetUrl: String) {
        guard let sourceUrl = URL(string: targetUrl) else {
            downloadCustomJsonButtonHelperResult = false
            return
        }
    }
    
    /* Alert message to show when the user tries to complete an in-app purchase when they already completed an in-app purchase */
    func inAppPurchaseNotPossible() {
        let alertController = UIAlertController(title: "In-App Purchase Error", message: "It looks like your account is unable to complete the in-app purchase.", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    func inAppPurchaseError(paymentFailedMsg: String) {
        let alertController = UIAlertController(title: "In-App Purchase Error", message: "The in-app purchase could not be completed for the following reason: \(paymentFailedMsg)", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    func inAppPurchaseAlreadyCompleted() {
        let alertController = UIAlertController(title: "In-App Purchase Completed", message: "The in-app purchase has already been completed.  If settings on the Settings page are not unlocked, try closing and re-starting this app.", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Alert message to show when a setting function is attempted to be used when in-app purchase has not been completed */
    func settingNotEnabled() {
        let alertController = UIAlertController(title: "Setting Locked", message: "That setting is currently locked.  Please see the section 'Unlock ALL Settings' for information on how to unlock settings.", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Alert message to show when resetting the customjson.json file with the default version */
    func foundResetSuccess() {
        let alertController = UIAlertController(title: "Operation Completed", message: "The operation completed successfully.  If the app had already been set to use custom JSON, the app should be closed and restarted so that it is using the newly reset custom JSON and not the version already loaded in memory.", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Alert message to show when a URL provided by the user is invalid */
    func mentionUrlInvalid() {
        let alertController = UIAlertController(title: "Error Found", message: "The URL that was specified appears to be invalid.", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Close", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Alert message to let user know that the asynchronous operation is about to start and they should check the documents folder */
    func mentionLoadStart() {
        let alertController = UIAlertController(title: "Download Custom JSON", message: "The download of custom JSON from the URL specified will begin and may take several seconds depending on connectivity and availability of the URL.  If the JSON from the URL is valid and in the correct structure, it will be placed into customjson.json in this app's Documents folder overwriting any data that may have already existed in customjson.json.  You could access customjson.json such as with iTunes or Finder to ensure that all of the data was downloaded completely.  In order to use that customjson.json in future app sessions, ensure that 'Use Custom JSON File' is ON and you have closed and restarted this app.", preferredStyle: .alert)
        let defaultAction = UIAlertAction(title: "Continue", style: .default, handler: nil)
        alertController.addAction(defaultAction)
        present(alertController, animated: true, completion: nil)
    }
    
    /* Terms and Conditions acceptance check */
    func didAcceptToc() -> Bool {
        var acceptedResult: Bool = false
        /* Reference to configuration data */
        var Obj_ConfigClass = ConfigClass()
        /* Check to see if TOC was accepted */
        Obj_ConfigClass.recallToc()
        /* TOC was accepted by the user */
        if Obj_ConfigClass.tocAccepted == true {
            acceptedResult = true
        }
        return acceptedResult
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        
        /* Setup in-app purchase observer */
        SKPaymentQueue.default().add(self)
        
        /* Google Ads - Configuration and invocation for this controller */
        /*
        Obj_ConfigClass.hasPurchasedAdRemoval()
        if (Obj_ConfigClass.showAds == true) {
            /* Show Ads */
            bannerView.adSize = kGADAdSizeBanner
            bannerView.adUnitID = Obj_ConfigClass.googleBannerAdUnitID
            bannerView.rootViewController = self
            bannerView.load(GADRequest())
            /* Uncomment for troubleshooting banner ad retrieval issues */
            //bannerView.delegate = self
        }
        */
        
        if didAcceptToc() == true {
            /* Set toggle switch state based on what has been saved already in userdefaults.standard */
            /* Reference to configuration data */
            var Obj_ConfigClass = ConfigClass()
            Obj_ConfigClass.recallUseCustomJsonValue()
            if Obj_ConfigClass.usecustomjson == true {
                useCustomJsonToggleManuallySet.isOn = true
            }
            else {
                useCustomJsonToggleManuallySet.isOn = false
            }
        }
        else {
            useCustomJsonToggleManuallySet.isOn = false
        }
    }
}

/* Google Ads - Delegate primarily for troubleshooting */
/*
extension SecondViewController: GADBannerViewDelegate {
    func adViewDidReceiveAd(_ bannerView: GADBannerView) {
        print("Received Ad")
    }
    func adView(_ bannerView: GADBannerView, didFailToReceiveAdWithError error: GADRequestError) {
        print("Banner Retrival Error \(error)")
    }
}
*/
